@@ -1,5 +1,9 @@
+from __future__ import division, print_function, absolute_import
 
 def configuration(parent_package='',top_path=None):
+    from scipy._build_utils.system_info import get_info, NotFoundError
+    lapack_opt = get_info("lapack_opt")
+
     from numpy.distutils.misc_util import Configuration
     config = Configuration('scipy',parent_package,top_path)
     config.add_subpackage('cluster')
@@ -8,10 +12,8 @@ def configuration(parent_package='',top_path=None):
     config.add_subpackage('integrate')
     config.add_subpackage('interpolate')
     config.add_subpackage('io')
-    config.add_subpackage('lib')
     config.add_subpackage('linalg')
-    config.add_subpackage('linsolve')
-    config.add_subpackage('maxentropy')
+    config.add_data_files('*.pxd')
     config.add_subpackage('misc')
     config.add_subpackage('odr')
     config.add_subpackage('optimize')
@@ -21,12 +23,12 @@ def configuration(parent_package='',top_path=None):
     config.add_subpackage('special')
     config.add_subpackage('stats')
     config.add_subpackage('ndimage')
-    config.add_subpackage('stsci')
-    config.add_subpackage('weave')
-    config.make_svn_version_py()  # installs __svn_version__.py
+    config.add_subpackage('_build_utils')
+    config.add_subpackage('_lib')
     config.make_config_py()
     return config
 
+
 if __name__ == '__main__':
     from numpy.distutils.core import setup
     setup(**configuration(top_path='').todict())
