@@ -159,12 +159,14 @@ Discrete distributions
    randint           -- Discrete Uniform
    skellam           -- Skellam
    zipf              -- Zipf
+   yulesimon         -- Yule-Simon
 
-Statistical functions
-=====================
+An overview of statistical functions is given below.
+Several of these functions have a similar version in
+`scipy.stats.mstats` which work for masked arrays.
 
-Several of these functions have a similar version in scipy.stats.mstats
-which work for masked arrays.
+Summary statistics
+==================
 
 .. autosummary::
    :toctree: generated/
@@ -173,12 +175,9 @@ which work for masked arrays.
    gmean             -- Geometric mean
    hmean             -- Harmonic mean
    kurtosis          -- Fisher or Pearson kurtosis
-   kurtosistest      --
    mode              -- Modal value
    moment            -- Central moment
-   normaltest        --
    skew              -- Skewness
-   skewtest          --
    kstat             --
    kstatvar          --
    tmean             -- Truncated arithmetic mean
@@ -190,6 +189,14 @@ which work for masked arrays.
    variation         -- Coefficient of variation
    find_repeats
    trim_mean
+   iqr
+   sem
+   bayes_mvs
+   mvsdist
+   entropy
+
+Frequency statistics
+====================
 
 .. autosummary::
    :toctree: generated/
@@ -207,23 +214,8 @@ which work for masked arrays.
    binned_statistic_2d  -- Compute a 2-D binned statistic for a set of data.
    binned_statistic_dd  -- Compute a d-D binned statistic for a set of data.
 
-.. autosummary::
-   :toctree: generated/
-
-   obrientransform
-   bayes_mvs
-   mvsdist
-   sem
-   zmap
-   zscore
-   iqr
-
-.. autosummary::
-   :toctree: generated/
-
-   sigmaclip
-   trimboth
-   trim1
+Correlation functions
+=====================
 
 .. autosummary::
    :toctree: generated/
@@ -235,8 +227,12 @@ which work for masked arrays.
    kendalltau
    weightedtau
    linregress
+   siegelslopes
    theilslopes
 
+Statistical tests
+=================
+
 .. autosummary::
    :toctree: generated/
 
@@ -255,6 +251,7 @@ which work for masked arrays.
    wilcoxon
    kruskal
    friedmanchisquare
+   brunnermunzel
    combine_pvalues
    jarque_bera
 
@@ -271,6 +268,12 @@ which work for masked arrays.
    fligner
    median_test
    mood
+   skewtest
+   kurtosistest
+   normaltest
+
+Transformations
+===============
 
 .. autosummary::
    :toctree: generated/
@@ -278,8 +281,18 @@ which work for masked arrays.
    boxcox
    boxcox_normmax
    boxcox_llf
+   yeojohnson
+   yeojohnson_normmax
+   yeojohnson_llf
+   obrientransform
+   sigmaclip
+   trimboth
+   trim1
+   zmap
+   zscore
 
-   entropy
+Statistical distances
+=====================
 
 .. autosummary::
    :toctree: generated/
@@ -287,6 +300,14 @@ which work for masked arrays.
    wasserstein_distance
    energy_distance
 
+Random variate generation
+=========================
+
+.. autosummary::
+   :toctree: generated/
+
+   rvs_ratio_uniforms
+
 Circular statistical functions
 ==============================
 
@@ -318,6 +339,7 @@ Plot-tests
    ppcc_plot
    probplot
    boxcox_normplot
+   yeojohnson_normplot
 
 
 Masked statistics functions
